import static org.junit.Assert.*;
import gcs.api.VehicleControllerFactoryImpl;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import jp.go.aist.rtm.RTC.Manager;
import jp.go.aist.rtm.RTC.port.OutPort;
import jp.go.aist.rtm.RTC.util.StringHolder;
import ms.Gps;
import ms.Home;
import ms.Imu;
import ms.Sys;
import ms.Uav;
import ms.aif.Connection;
import ms.aif.TelemetryListener;
import ms.aif.VehicleController;
import ms.aif.VehicleControllerFactory;
import ms.aif.command.Control;
import ms.aif.command.ControlCommand;
import ms.aif.command.SingleWayPoint;
import ms.aif.command.WayPointCommand;
import ms.aif.response.CommandResponse;
import ms.aif.response.Status;
import ms.aif.response.TelemetryResponse;

import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.api.support.membermodification.MemberMatcher;

import com.sun.org.apache.xpath.internal.operations.Mod;

import static org.mockito.Mockito.*;
import static org.mockito.Matchers.*;
import static org.powermock.api.mockito.PowerMockito.verifyPrivate;
import RTC.GPSData;
import RTC.GPSTime;
import RTC.Orientation3D;
import RTC.ReturnCode_t;
import RTC.Time;
import RTC.TimedFloat;
import RTC.TimedFloatSeq;
import RTC.TimedOrientation3D;
import RTC.TimedString;

public class MiniSurveyorIOImplTest {
	public static Object getPrivateField(Object target_object, String field_name)
			throws Exception {
		Class c = target_object.getClass();
		Field fld = c.getDeclaredField(field_name);
		fld.setAccessible(true);

		return fld.get(target_object);
	}

	public static Object doPrivateMethod(Object target_object,
			String field_name, Class[] classArray, Object[] args)
			throws Exception {
		Class c = target_object.getClass();
		Method method = c.getDeclaredMethod(field_name, classArray);
		method.setAccessible(true);

		return method.invoke(target_object, args);
	}

	public static Object doPrivateMethod(Object target_object, String field_name)
			throws Exception {
		Class c = target_object.getClass();
		Method method = c.getDeclaredMethod(field_name);
		method.setAccessible(true);

		return method.invoke(target_object);
	}

	public static void setPrivateField(Object target_object, String field_name,
			Object value) throws Exception {
		Class c = target_object.getClass();
		Field fld = c.getDeclaredField(field_name);
		fld.setAccessible(true);
		fld.set(target_object, value);
	}

	@Test
	public void testOnStartupInt() throws Exception {

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Boolean[] ins1 = { false, true, true, true };
		Boolean[] ins2 = { null, false, true, true };
		Boolean[] ins3 = { null, null, false, true };

		ReturnCode_t[] ret = { ReturnCode_t.RTC_ERROR, ReturnCode_t.RTC_ERROR,
				ReturnCode_t.RTC_ERROR, ReturnCode_t.RTC_OK };
		int[] retCnt1 = { 1, 1, 1, 1 };
		int[] retCnt2 = { 0, 1, 1, 1 };
		int[] retCnt3 = { 0, 0, 1, 0 };
		int[] retCnt4 = { 0, 0, 1, 1 };

		int j1 = 0;
		int j2 = 0;
		int j3 = 0;
		int j4 = 0;

		Class[] cls1 = { long.class };
		Class[] cls2 = { String.class };

		Long[] param1 = { (long) 1000 };
		String[] param2 = { (String) getPrivateField(msio, "ERR_MSG_1") };

		for (int i = 0; i < ins1.length; i++) {
			if (i != 0) {
				assertEquals(j1, retCnt1[i - 1]);
				assertEquals(j2, retCnt2[i - 1]);
				assertEquals(j3, retCnt3[i - 1]);
				assertEquals(j4, retCnt4[i - 1]);
			}
			j1 = 0;
			j2 = 0;
			j3 = 0;
			j4 = 0;
			doPrivateMethod(msio, "getAPIService");
			j1++;
			if (ins1[i] == false) {
				assertEquals(ReturnCode_t.RTC_ERROR, ret[i]);
				continue;
			}

			doPrivateMethod(msio, "getTelemetryListener");
			j2++;
			if (ins2[i] == false) {
				assertEquals(ReturnCode_t.RTC_ERROR, ret[i]);
				continue;
			}

			doPrivateMethod(msio, "startSleep", cls1, param1);
			j4++;

			if (ins3[i] == false) {
				doPrivateMethod(msio, "errMsgWrite", cls2, param2);
				j3++;
				assertEquals(ReturnCode_t.RTC_ERROR, ret[i]);
				continue;
			}

			assertEquals(ReturnCode_t.RTC_OK, ret[i]);

		}

		assertEquals(j1, retCnt1[3]);
		assertEquals(j2, retCnt2[3]);
		assertEquals(j3, retCnt3[3]);
		assertEquals(j4, retCnt4[3]);
	}

	@Test
	public void testOnActivatedInt() throws Exception {

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Boolean[] ins1 = { true, true, true, false };
		Boolean[] ins2 = { false, true, true };
		Boolean[] ins3 = { null, false, true };

		ReturnCode_t[] ret = { ReturnCode_t.RTC_ERROR, ReturnCode_t.RTC_ERROR,
				ReturnCode_t.RTC_OK, msio.on_startup(0) };
		int[] retCnt1 = { 1, 0, 0, 0 };
		int[] retCnt2 = { 0, 1, 0, 0 };
		int[] retCnt3 = { 0, 0, 1, 0 };
		int[] retCnt4 = { 0, 0, 1, 0 };

		int j1 = 0;
		int j2 = 0;
		int j3 = 0;
		int j4 = 0;

		Class[] cls1 = { String.class };
		Class[] cls2 = { String.class };
		Class[] cls3 = { int.class };

		String[] param1 = { (String) getPrivateField(msio, "ERR_MSG_2") };
		String[] param2 = { (String) getPrivateField(msio, "ERR_MSG_3") };
		Integer[] param3 = { (Integer) getPrivateField(msio, "STS_ARMED") };

		for (int i = 0; i < ins1.length; i++) {
			if (i != 0) {
				assertEquals(j1, retCnt1[i - 1]);
				assertEquals(j2, retCnt2[i - 1]);
				assertEquals(j3, retCnt3[i - 1]);
				assertEquals(j4, retCnt4[i - 1]);
			}
			j1 = 0;
			j2 = 0;
			j3 = 0;
			j4 = 0;
			if (ins1[i] == false) {
				assertEquals(msio.on_startup(0), ret[i]);
				continue;
			}
			if (ins2[i] == false) {
				doPrivateMethod(msio, "errMsgWrite", cls1, param1);
				j1++;
				assertEquals(ReturnCode_t.RTC_ERROR, ret[i]);
				continue;
			}
			if (ins3[i] == false) {
				doPrivateMethod(msio, "errMsgWrite", cls2, param2);
				j2++;
				assertEquals(ReturnCode_t.RTC_ERROR, ret[i]);
				continue;
			}
			doPrivateMethod(msio, "flightTimeStart");
			j3++;
			doPrivateMethod(msio, "changeDroneSts", cls3, param3);
			j4++;

			assertEquals(ReturnCode_t.RTC_OK, ret[i]);
		}
		assertEquals(j1, retCnt1[ins1.length - 1]);
		assertEquals(j2, retCnt2[ins1.length - 1]);
		assertEquals(j3, retCnt3[ins1.length - 1]);
		assertEquals(j4, retCnt4[ins1.length - 1]);
	}

	@Test
	public void testOnExecuteInt() throws Exception {

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Boolean[] ins1 = { false, true, true, true, true, true, true, true,
				true, true, true, true, true, true, true, true };
		Boolean[] ins2 = { null, true, true, true, true, true, true, true,
				true, true, true, true, true, false, false, false };
		Boolean[] ins3 = { null, true, true, false, false, true, true, false,
				false, true, true, false, false };
		Short land = (Short) getPrivateField(msio, "FM_LAND");
		Short nodata = (Short) getPrivateField(msio, "FM_NO_DATA");
		Short[] ins4 = { null, nodata, land, nodata, land, nodata, land,
				nodata, land, nodata, land, nodata, land };
		Boolean[] ins5 = { null, true, true, true, true, false, false, false,
				false, false, false, false, false, true, false, false };
		short[] ins6 = { 0, 0, 0, 0, 0, 30, 30, 30, 30, 0, 0, 0, 0, 0, 30, 0 };

		ReturnCode_t[] ret = { ReturnCode_t.RTC_ERROR, ReturnCode_t.RTC_ERROR,
				ReturnCode_t.RTC_OK, msio.on_startup(0) };
		int[] retCnt1 = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int[] retCnt2 = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int[] retCnt3 = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int[] retCnt4 = { 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0 };
		int[] retCnt5 = { 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
		int[] retCnt6 = { 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
		int[] retCnt7 = { 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0 };
		int[] retCnt8 = { 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0 };
		int[] retCnt9 = { 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0 };
		int[] retCnt10 = { 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0 };
		int[] retCnt11 = { 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
		int[] retCnt12 = { 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };

		int j1 = 0;
		int j2 = 0;
		int j3 = 0;
		int j4 = 0;
		int j5 = 0;
		int j6 = 0;
		int j7 = 0;
		int j8 = 0;
		int j9 = 0;
		int j10 = 0;
		int j11 = 0;
		int j12 = 0;

		Class[] cls1 = { int.class };
		Class[] cls2 = { String.class };
		Class[] cls3 = { int.class };
		Class[] cls4 = { int.class };

		Integer[] param1 = { (Integer) getPrivateField(msio, "STS_DISARM") };
		short time = (short) getPrivateField(msio, "comm_err_timer");
		String[] param2 = { (String) getPrivateField(msio, "ERR_MSG_4") };
		Integer[] param3 = { (Integer) getPrivateField(msio, "STS_RTL") };
		Integer[] param4 = { (Integer) doPrivateMethod(msio, "flightTimeCount") };
		for (int i = 0; i < ins1.length; i++) {
			if (i != 0) {
				assertEquals(j1, retCnt1[i - 1]);
				assertEquals(j2, retCnt2[i - 1]);
				assertEquals(j3, retCnt3[i - 1]);
				assertEquals(j4, retCnt4[i - 1]);
				assertEquals(j5, retCnt5[i - 1]);
				assertEquals(j6, retCnt6[i - 1]);
				assertEquals(j7, retCnt7[i - 1]);
				assertEquals(j8, retCnt8[i - 1]);
				assertEquals(j9, retCnt9[i - 1]);
				assertEquals(j10, retCnt10[i - 1]);
				assertEquals(j11, retCnt11[i - 1]);
				assertEquals(j12, retCnt12[i - 1]);
			}
			j1 = 0;
			j2 = 0;
			j3 = 0;
			j4 = 0;
			j5 = 0;
			j6 = 0;
			j7 = 0;
			j8 = 0;
			j9 = 0;
			j10 = 0;
			j11 = 0;
			j12 = 0;
			if (ins1[i] == false) {
				doPrivateMethod(msio, "changeDroneSts", cls1, param1);
				j1++;
				msio.m_outStateOut.write();
				j2++;
				msio.deactivate(0);
				j3++;
				continue;
			}

			if (ins2[i]) {
				if (ins3[i] && ins4[i] != land) {
					doPrivateMethod(msio, "startLand");
					j4++;
				}
			}

			doPrivateMethod(msio, "drvStateSelect");
			j5++;
			doPrivateMethod(msio, "fltCtrlSelect");
			j6++;

			if (ins5[i]) {
				setPrivateField(msio, "comm_err_timer", (short) 0);
				assertEquals((short) 0, getPrivateField(msio, "comm_err_timer"));
				doPrivateMethod(msio, "flightCtrlCore");
				j7++;
			} else {
				setPrivateField(msio, "comm_err_timer", (short) (time + 1));
				assertEquals((short) 1, getPrivateField(msio, "comm_err_timer"));
				if (ins6[i] == (short) getPrivateField(msio, "COMM_ERR_TIMEUP")) {
					setPrivateField(msio, "comm_err_timer", (short) 0);
					assertEquals((short) 0,
							getPrivateField(msio, "comm_err_timer"));
					doPrivateMethod(msio, "errMsgWrite", cls2, param2);
					j8++;
					doPrivateMethod(msio, "changeDroneSts", cls3, param3);
					j9++;
					doPrivateMethod(msio, "startGOHOME");
					j10++;
				}
			}

			doPrivateMethod(msio, "setFlightTime", cls4, param4);
			j11++;
			doPrivateMethod(msio, "portWrite");
			j12++;

		}
		assertEquals(j1, retCnt1[ins1.length - 1]);
		assertEquals(j2, retCnt2[ins1.length - 1]);
		assertEquals(j3, retCnt3[ins1.length - 1]);
		assertEquals(j4, retCnt4[ins1.length - 1]);
		assertEquals(j5, retCnt5[ins1.length - 1]);
		assertEquals(j6, retCnt6[ins1.length - 1]);
		assertEquals(j7, retCnt7[ins1.length - 1]);
		assertEquals(j8, retCnt8[ins1.length - 1]);
		assertEquals(j9, retCnt9[ins1.length - 1]);
		assertEquals(j10, retCnt10[ins1.length - 1]);
		assertEquals(j11, retCnt11[ins1.length - 1]);
		assertEquals(j12, retCnt12[ins1.length - 1]);
	}

	@Test
	public void testGetConnection() throws Exception {
		Connection ecpected = new Connection("COM3", 115200, 0, 0, 0, true,
				3000);

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Class[] classArray = { String.class };
		String[] param = { "COM3" };

		Connection actual = (Connection) doPrivateMethod(msio, "getConnection",
				classArray, param);

		VehicleControllerFactory vfactory = new VehicleControllerFactoryImpl();
		VehicleController vController = vfactory
				.createVehicleController(ecpected);

		VehicleControllerFactory vfactory2 = new VehicleControllerFactoryImpl();
		VehicleController vController2 = vfactory2
				.createVehicleController(actual);

		assertTrue(ecpected.getConnectionStatus().getErrorMessage()
				.equals(actual.getConnectionStatus().getErrorMessage()));
	}

	@Test
	public void testGetAPIService() throws Exception {

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		msio.m_ConnectPort.value = "COM3";
		doPrivateMethod(msio, "getAPIService");

		assertNotNull(getPrivateField(msio, "con"));
		assertNotNull(getPrivateField(msio, "vController"));

	}

	@Test
	public void testCheckConnectStatus() throws Exception {

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Connection con = spy(new Connection("COM3", 115200, 0, 0, 0, true, 3000));
		Status st = mock(Status.class);

		Class[] cls1 = { Connection.class };
		Connection[] param1 = { con };

		Integer[] ins1 = { 2, 1, 2, 1 };
		Integer[] ins2 = { 2, 2, 1, 1 };

		Boolean[] ret1 = { false, false, false, true };

		con.setConnectionStatus(st);

		for (int i = 0; i < ins1.length; i++) {
			PowerMockito.when(con.getConnectionStatus(), "getErrorId")
					.thenReturn(ins1[i]);
			PowerMockito.when(con.getConnectionStatus(), "getErrorCategory")
					.thenReturn(ins2[i]);

			assertEquals(
					doPrivateMethod(msio, "checkConnectStatus", cls1, param1),
					ret1[i]);
		}

	}

	@Test
	public void testGetTelemetryListener() throws Exception {

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Connection con = mock(Connection.class);

		VehicleControllerFactory vfactory = new VehicleControllerFactoryImpl();
		VehicleController vController = vfactory.createVehicleController(con);

		VehicleControllerFactory vfactory2 = new VehicleControllerFactoryImpl();
		VehicleController vController2 = vfactory2.createVehicleController(con);

		setPrivateField(msio, "vController", vController);

		Class[] cls1 = { Uav.class };
		Class[] cls2 = { String.class };

		Status sta = vController.setTelemetryListener(new TelemetryListener() {
			int j1 = 0;
			int j2 = 0;
			Status st_past = null;

			@Override
			public void onUpdate(TelemetryResponse telemetryResponse) {
				try {
					setPrivateField(msio, "uav", telemetryResponse
							.getTelemetry().getUav());
					assertEquals(getPrivateField(msio, "uav"),
							telemetryResponse.getTelemetry().getUav());
				} catch (Exception e) {
					// TODO 自動生成された catch ブロック
					e.printStackTrace();
				}
				try {
					doPrivateMethod(msio, "telemetryEvent", cls1,
							new Uav[] { (Uav) getPrivateField(msio, "uav") });
					j1++;
				} catch (Exception e) {
					// TODO 自動生成された catch ブロック
					e.printStackTrace();
				}
				assertEquals(j1, 1);

				Status status = telemetryResponse.getStatus();
				if (status != st_past) {
					try {
						doPrivateMethod(getPrivateField(msio, "LOGGER"),
								"info", cls2,
								new String[] { status.getErrorMessage() });
						j2++;
					} catch (Exception e) {
						// TODO 自動生成された catch ブロック
						e.printStackTrace();
					}
				}
				assertEquals(j2, 1);
				st_past = status;
			}
		});
		assertNotNull(sta);

		Status sta2 = vController2
				.setTelemetryListener(new TelemetryListener() {
					int j1 = 0;
					int j2 = 0;
					Status st_past = null;

					@Override
					public void onUpdate(TelemetryResponse telemetryResponse) {
						try {
							setPrivateField(msio, "uav", telemetryResponse
									.getTelemetry().getUav());
							assertEquals(getPrivateField(msio, "uav"),
									telemetryResponse.getTelemetry().getUav());
						} catch (Exception e) {
							// TODO 自動生成された catch ブロック
							e.printStackTrace();
						}
						try {
							doPrivateMethod(
									msio,
									"telemetryEvent",
									cls1,
									new Uav[] { (Uav) getPrivateField(msio,
											"uav") });
							j1++;
						} catch (Exception e) {
							// TODO 自動生成された catch ブロック
							e.printStackTrace();
						}
						assertEquals(j1, 1);

						Status status = st_past;
						if (status != st_past) {
							try {
								doPrivateMethod(
										getPrivateField(msio, "LOGGER"),
										"info",
										cls2,
										new String[] { status.getErrorMessage() });
								j2++;
							} catch (Exception e) {
								// TODO 自動生成された catch ブロック
								e.printStackTrace();
							}
						}
						assertEquals(j2, 0);
						st_past = status;
					}
				});
		assertNotNull(sta2);
	}

	@Test
	public void testTelemetryEvent() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Uav uav = mock(Uav.class);

		Class[] cls1 = { Uav.class };
		Uav[] param1 = { uav };

		int j = 0;

		doPrivateMethod(
				msio,
				"setGPSHome",
				new Class[] { GPSData.class },
				new GPSData[] { (GPSData) doPrivateMethod(msio, "readGPSHome",
						cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setGPSDrone",
				new Class[] { GPSData.class },
				new GPSData[] { (GPSData) doPrivateMethod(msio, "readGPSDrone",
						cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setVehicleMode",
				new Class[] { TimedString.class },
				new TimedString[] { (TimedString) doPrivateMethod(msio,
						"readVehicleMode", cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setVoltage",
				new Class[] { TimedFloat.class },
				new TimedFloat[] { (TimedFloat) doPrivateMethod(msio,
						"readVoltage", cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setGyro",
				new Class[] { TimedOrientation3D.class },
				new TimedOrientation3D[] { (TimedOrientation3D) doPrivateMethod(
						msio, "readGyro", cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setAccel",
				new Class[] { TimedFloatSeq.class },
				new TimedFloatSeq[] { (TimedFloatSeq) doPrivateMethod(msio,
						"readAccel", cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setMag",
				new Class[] { TimedFloat.class },
				new TimedFloat[] { (TimedFloat) doPrivateMethod(msio,
						"readMag", cls1, param1) });
		j++;
		doPrivateMethod(
				msio,
				"setDrv",
				new Class[] { Short.class },
				new Short[] { (Short) doPrivateMethod(msio, "readDrv", cls1,
						param1) });
		assertEquals(j, 7);
	}

	@Test
	public void testCheckTelemetryStatus() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Status sta = mock(Status.class);

		Class[] cls1 = { Status.class };
		Status[] param1 = { sta };

		Integer[] ins1 = { 1, 1, 3, 3 };
		Integer[] ins2 = { 1, 3, 1, 3 };

		Boolean[] ret1 = { false, false, true, false };

		for (int i = 0; i < ins1.length; i++) {
			PowerMockito.when(sta, "getErrorId").thenReturn(ins1[i]);
			PowerMockito.when(sta, "getErrorCategory").thenReturn(ins2[i]);

			assertEquals(
					doPrivateMethod(msio, "checkTelemetryStatus", cls1, param1),
					ret1[i]);
		}
	}

	@Test
	public void testCheckGPSHome() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Double db = Double.NaN;

		Double[] ins1 = { 1.0, 1.0, 1.0, 1.0, db.NaN, db.NaN, db.NaN, db.NaN };
		Double[] ins2 = { 1.0, db, 1.0, db.NaN, 1.0, db.NaN, 1.0, db.NaN };
		Double[] ins3 = { 1.0, 1.0, db.NaN, db.NaN, 1.0, 1.0, db.NaN, db.NaN };

		Boolean[] ret1 = { true, false, false, false, false, false, false,
				false };

		for (int i = 0; i < ins1.length; i++) {
			msio.m_outGPSHome_val.latitude = ins1[i];
			msio.m_outGPSHome_val.longitude = ins2[i];
			msio.m_outGPSHome_val.altitude = ins3[i];
			assertEquals(doPrivateMethod(msio, "checkGPSHome"), ret1[i]);
		}
	}

	@Test
	public void testCheckArmed() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		short dis = (short) getPrivateField(msio, "FM_DISARM");
		short amd = (short) getPrivateField(msio, "FM_ARMED");
		Short[] ins1 = { dis, amd };

		Boolean[] ret1 = { false, true };

		for (int i = 0; i < ins1.length; i++) {
			setPrivateField(msio, "drv_state", ins1[i].shortValue());
			assertEquals(doPrivateMethod(msio, "checkArmed"), ret1[i]);
		}
	}

	@Test
	public void testFlightTimeStart() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		doPrivateMethod(msio, "flightTimeStart");
		assertNotEquals(getPrivateField(msio, "flight_time_start"), 0);

	}

	@Test
	public void testStartGOHOME() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Connection con = mock(Connection.class);
		VehicleControllerFactory vfactory = new VehicleControllerFactoryImpl();
		VehicleController vController = vfactory.createVehicleController(con);
		setPrivateField(msio, "vController", vController);

		Control cmd = Control.GOHOME;

		Control command = Control.GOHOME;
		ControlCommand ccmd = new ControlCommand() {
			@Override
			public Control getControl() {
				return command;
			}
		};
		CommandResponse cmdRes = vController.sendCommand(ccmd);
		assertEquals(ccmd.getControl(), cmd);

	}

	@Test
	public void testCheckHomeArrival() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Double[] ins1 = { 1.0, 1.1 };

		Boolean[] ret1 = { true, false };

		for (int i = 0; i < ins1.length; i++) {
			Boolean ret = false;
			if (ins1[i] <= (Double) getPrivateField(msio, "HOME_ARRIVAL_VAL")) {
				ret = true;
			}
			assertEquals(ret, ret1[i]);
		}

	}

	@Test
	public void testStartLand() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Connection con = mock(Connection.class);
		VehicleControllerFactory vfactory = new VehicleControllerFactoryImpl();
		VehicleController vController = vfactory.createVehicleController(con);
		setPrivateField(msio, "vController", vController);

		Control cmd = Control.LAND;

		Control command = Control.LAND;
		ControlCommand ccmd = new ControlCommand() {
			@Override
			public Control getControl() {
				return command;
			}
		};
		CommandResponse cmdRes = vController.sendCommand(ccmd);
		assertEquals(ccmd.getControl(), cmd);

	}

	@Test
	public void testStartRTL() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		msio.on_startup(0);

		int dis = (int) getPrivateField(msio, "STS_DISARM");
		int amd = (int) getPrivateField(msio, "STS_ARMED");
		int rtl = (int) getPrivateField(msio, "STS_RTL");
		int hov = (int) getPrivateField(msio, "STS_HOVERING");

		Class[] cls1 = { int.class };
		Class[] cls2 = { long.class };

		Integer[] param1 = { rtl };
		Long[] param2 = { (long) 1000 };

		int[] ins1 = { dis, amd, rtl, rtl, hov, hov };
		Boolean[] ins2 = { null, null, true, false, true, false };

		int[] retCnt1 = { 0, 0, 0, 0, 1, 1 };
		int[] retCnt2 = { 0, 0, 1, 1, 1, 1 };
		int[] retCnt3 = { 0, 0, 0, 0, 1, 1 };
		int[] retCnt4 = { 0, 0, 0, 1, 0, 1 };
		int[] retCnt5 = { 0, 0, 0, 1, 0, 1 };
		int[] retCnt6 = { 0, 0, 1, 1, 1, 1 };
		int[] retCnt7 = { 0, 0, 1, 1, 1, 1 };

		int j1 = 0;
		int j2 = 0;
		int j3 = 0;
		int j4 = 0;
		int j5 = 0;
		int j6 = 0;
		int j7 = 0;

		for (int i = 0; i < ins1.length; i++) {
			j1 = 0;
			j2 = 0;
			j3 = 0;
			j4 = 0;
			j5 = 0;
			j6 = 0;
			j7 = 0;

			if (dis != ins1[i] && amd != ins1[i] && rtl != ins1[i]) {
				doPrivateMethod(msio, "changeDroneSts", cls1, param1);
				j1++;
				msio.m_outStateOut.write();
				j2++;
				doPrivateMethod(msio, "startGOHOME");
				j3++;
				while (ins2[i] == false) {
					doPrivateMethod(msio, "startSleep", cls2, param2);
					j4++;
					msio.m_outVehicleModeOut.write();
					j5++;
					break;
				}
				doPrivateMethod(msio, "startLand");
				j6++;
				doPrivateMethod(msio, "changeDisarm");
				j7++;
			} else if (rtl == ins1[i]) {
				msio.m_outStateOut.write();
				j2++;
				while (ins2[i] == false) {
					doPrivateMethod(msio, "startSleep", cls2, param2);
					j4++;
					msio.m_outVehicleModeOut.write();
					j5++;
					break;
				}
				doPrivateMethod(msio, "startLand");
				j6++;
				doPrivateMethod(msio, "changeDisarm");
				j7++;
			}
			assertEquals(j1, retCnt1[i]);
			assertEquals(j2, retCnt2[i]);
			assertEquals(j3, retCnt3[i]);
			assertEquals(j4, retCnt4[i]);
			assertEquals(j5, retCnt5[i]);
			assertEquals(j6, retCnt6[i]);
			assertEquals(j7, retCnt7[i]);
		}
	}

	@Test
	public void testChangeDisarm() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		short dis = (short) getPrivateField(msio, "FM_DISARM");
		short amd = (short) getPrivateField(msio, "FM_ARMED");

		Class[] cls1 = { long.class };
		Class[] cls2 = { int.class };

		Long[] param1 = { (long) 1000 };
		Short[] param2 = { dis };
		int[] ins1 = { dis, amd };

		int[] retCnt1 = { 0, 1 };
		int[] retCnt2 = { 0, 1 };
		int[] retCnt3 = { 1, 1 };
		int[] retCnt4 = { 1, 1 };

		int j1 = 0;
		int j2 = 0;
		int j3 = 0;
		int j4 = 0;

		for (int i = 0; i < 2; i++) {
			j1 = 0;
			j2 = 0;
			j3 = 0;
			j4 = 0;

			while (ins1[i] != dis) {
				doPrivateMethod(msio, "startSleep", cls1, param1);
				j1++;
				msio.m_outVehicleModeOut.write();
				j2++;
				break;
			}
			doPrivateMethod(msio, "changeDroneSts", cls2, param2);
			j3++;
			msio.m_outStateOut.write();
			j4++;

			assertEquals(j1, retCnt1[i]);
			assertEquals(j2, retCnt2[i]);
			assertEquals(j3, retCnt3[i]);
			assertEquals(j4, retCnt4[i]);
		}
	}

	@Test
	public void testReadTargetPos() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Boolean[] ins1 = { true, false };

		int[] retCnt1 = { 1, 0 };
		Boolean[] ret1 = { true, false };

		int j1 = 0;

		for (int i = 0; i < ins1.length; i++) {
			j1 = 0;
			Boolean ret = false;
			if (ins1[i]) {
				msio.m_inTargetPosIn.read();
				j1++;
				ret = true;
			}

			assertEquals(ret, ret1[i]);
			assertEquals(j1, retCnt1[i]);
		}
	}

	@Test
	public void testCheckDroneSts() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		int dis = (int) getPrivateField(msio, "STS_DISARM");
		int amd = (int) getPrivateField(msio, "STS_ARMED");

		Class[] cls1 = { int.class };
		Integer[] param1 = { dis };
		Integer[] param2 = { amd };

		Integer[][] ins1 = { param1, param2 };

		doPrivateMethod(msio, "changeDroneSts", cls1, param1);

		Boolean[] ret1 = { true, false };

		for (int i = 0; i < ins1.length; i++) {
			Boolean ret = (Boolean) doPrivateMethod(msio, "checkDroneSts",
					cls1, ins1[i]);
			assertEquals(ret, ret1[i]);
		}

	}

	@Test
	public void testFlightCtrlCore() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		msio.on_startup(0);

		short dis = (short) getPrivateField(msio, "FM_DISARM");
		short flt = (short) getPrivateField(msio, "FM_FLIGHT");
		int rtl = (int) getPrivateField(msio, "STS_RTL");
		int off = (int) getPrivateField(msio, "STS_TAKEOFF");
		int hov = (int) getPrivateField(msio, "STS_HOVERING");

		Class[] cls1 = { int.class };
		Integer[] param1 = { rtl };

		Short[] ins1 = { flt, flt, flt, flt, dis, dis, dis, flt, flt, flt, dis,
				dis, dis };
		Boolean[] ins2 = { true, true, true, true, true, true, true, false,
				false, false, false, false, false };
		int[] ins3 = { hov, hov, rtl, off, rtl, off, rtl, off, hov, rtl, off,
				hov, rtl, off, hov };
		Boolean[] ins4 = { true, false };

		int[] retCnt1 = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int[] retCnt2 = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
		int[] retCnt3 = { 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

		int j1 = 0;
		int j2 = 0;
		int j3 = 0;

		for (int i = 0; i < ins1.length; i++) {
			j1 = 0;
			j2 = 0;
			j3 = 0;
			if (ins1[i] == flt && ins2[i] && ins3[i] != rtl && ins3[i] != off) {
				if (ins4[i]) {
					doPrivateMethod(msio, "startGOHOME");
					j1++;
					doPrivateMethod(msio, "changeDroneSts", cls1, param1);
					j2++;
				} else {
					doPrivateMethod(msio, "setTargetPos");
					j3++;
				}
			}
			assertEquals(j1, retCnt1[i]);
			assertEquals(j2, retCnt2[i]);
			assertEquals(j3, retCnt3[i]);
		}

	}

	@Test
	public void testHomeTargetPosSame() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		double[] ins1 = { 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0 };
		double[] ins2 = { 0.0, 0.0, 1.1, 1.1, 0.0, 0.0, 1.1, 1.1 };
		double[] ins3 = { 0.0, 1.2, 0.0, 1.2, 0.0, 1.2, 0.0, 1.2 };

		Boolean[] ret1 = { true, false, false, false, false, false, false,
				false };

		msio.m_outGPSHome_val.latitude = 0.0;
		msio.m_outGPSHome_val.longitude = 0.0;
		msio.m_outGPSHome_val.altitude = 0.0;

		for (int i = 0; i < ins1.length; i++) {
			msio.m_inTargetPos.v.latitude = ins1[i];
			msio.m_inTargetPos.v.longitude = ins2[i];
			msio.m_inTargetPos.v.altitude = ins3[i];
			Boolean ret = (Boolean) doPrivateMethod(msio, "homeTargetPosSame");
			assertEquals(ret, ret1[i]);
		}
	}

	@Test
	public void testStartPause() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Connection con = mock(Connection.class);
		VehicleControllerFactory vfactory = new VehicleControllerFactoryImpl();
		VehicleController vController = vfactory.createVehicleController(con);
		setPrivateField(msio, "vController", vController);

		Control cmd = Control.PAUSE;

		Control command = Control.PAUSE;
		ControlCommand ccmd = new ControlCommand() {
			@Override
			public Control getControl() {
				return command;
			}
		};
		CommandResponse cmdRes = vController.sendCommand(ccmd);
		assertEquals(ccmd.getControl(), cmd);

	}

	@Test
	public void testSetTargetPos() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		msio.on_startup(0);

		double[] ins1 = { 2, 2, 2, 3 };
		double[] ins2 = { 0, 1, 1 };
		Boolean[] ins3 = { null, false, true };

		Class[] cls1 = { int.class };
		Class[] cls2 = { int.class };
		int alt = (int) getPrivateField(msio, "STS_ALT_CHG");
		int hov = (int) getPrivateField(msio, "STS_HOVERING");
		int mov = (int) getPrivateField(msio, "STS_IN_MOVE");
		Integer[] param1 = { alt };
		Integer[] param2 = { hov };
		Integer[] param3 = { mov };

		double[] ret1 = { 1.0, 0.0, 1.0, 0.0 };
		msio.m_inTargetPos.v.altitude = 1.0;
		Integer[][] ret2 = { param2, param1, param1, param3 };
		double[] ret3 = { 0.0, 0.0, 0.0, 1.1 };
		msio.m_inTargetPos.v.latitude = 1.1;
		double[] ret4 = { 0.0, 0.0, 0.0, 1.2 };
		msio.m_inTargetPos.v.longitude = 1.2;

		int[] retCnt1 = { 0, 0, 1, 1 };
		int[] retCnt2 = { 1, 0, 0, 0 };

		int j1 = 0;
		int j2 = 0;

		for (int i = 0; i < ins1.length; i++) {
			setPrivateField(msio, "alt_chg_req", 0.0);
			setPrivateField(msio, "move_target_lat", 0.0);
			setPrivateField(msio, "move_target_lon", 0.0);
			j1 = 0;
			j2 = 0;
			if (ins1[i] <= (double) getPrivateField(msio, "MIN_MOVE_DIS")) {
				if (ins2[i] > 0) {
					if (ins3[i]) {
						doPrivateMethod(msio, "setSingleWayPoint");
						setPrivateField(msio, "alt_chg_req",
								msio.m_inTargetPos.v.altitude);
						j1++;
					}
					doPrivateMethod(msio, "changeDroneSts", cls1, param1);
				} else {
					doPrivateMethod(msio, "startPause");
					j2++;
					doPrivateMethod(msio, "changeDroneSts", cls1, param2);
					setPrivateField(msio, "alt_chg_req",
							msio.m_inTargetPos.v.altitude);
				}
			} else {
				setPrivateField(msio, "move_target_lat",
						msio.m_inTargetPos.v.latitude);
				setPrivateField(msio, "move_target_lon",
						msio.m_inTargetPos.v.longitude);
				doPrivateMethod(msio, "changeDroneSts", cls1, param3);
				doPrivateMethod(msio, "setSingleWayPoint");
				j1++;
			}
			assertEquals(
					(double) getPrivateField(msio, "alt_chg_req") == ret1[i],
					true);
			assertEquals(j1, retCnt1[i]);
			assertEquals(j2, retCnt2[i]);
			assertEquals(doPrivateMethod(msio, "checkDroneSts", cls2, ret2[i]),
					true);
			assertEquals(
					(double) getPrivateField(msio, "move_target_lat") == ret3[i],
					true);
			assertEquals(
					(double) getPrivateField(msio, "move_target_lon") == ret4[i],
					true);
		}
	}

	@Test
	public void testSetSingleWayPoint() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Connection con = mock(Connection.class);
		VehicleControllerFactory vfactory = new VehicleControllerFactoryImpl();
		VehicleController vController = vfactory.createVehicleController(con);
		setPrivateField(msio, "vController", vController);

		SingleWayPoint wayPoint = new SingleWayPoint(
				msio.m_inTargetPos.v.latitude, msio.m_inTargetPos.v.longitude,
				msio.m_inTargetPos.v.altitude, 0.0, (double) getPrivateField(
						msio, "MOVE_SPEED"));

		SingleWayPoint swp = wayPoint;
		WayPointCommand wpc = new WayPointCommand() {
			@Override
			public SingleWayPoint getWayPoint() {
				return swp;
			}
		};

		CommandResponse cmdRes = vController.sendCommand(wpc);

		assertEquals(wpc.getWayPoint(), wayPoint);

	}

	@Test
	public void testDrvStateSelect() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		int off = (int) getPrivateField(msio, "STS_TAKEOFF");
		int rtl = (int) getPrivateField(msio, "STS_RTL");
		short dis = (short) getPrivateField(msio, "FM_DISARM");
		short amd = (short) getPrivateField(msio, "FM_ARMED");
		short flt = (short) getPrivateField(msio, "FM_FLIGHT");

		Integer[] ins1 = { (int) dis, (int) amd, (int) flt, (int) flt,
				(int) flt };
		Boolean[] ins2 = { null, null, true, false, false };
		Boolean[] ins3 = { null, null, null, true, false };

		Integer[] ret1 = { (int) dis };
		Integer[] ret2 = { (int) amd };
		Integer[] ret3 = { (int) off };
		Integer[] ret4 = { (int) rtl };
		Integer[] ret5 = { 0 };
		Integer[][] rets1 = { ret1, ret2, ret3, ret4, ret5 };

		Class[] cls1 = { int.class };
		Integer[] param0 = { 0 };
		Integer[] param1 = { (int) dis };
		Integer[] param2 = { (int) amd };
		Integer[][] param12 = { param1, param2 };
		Integer[] param3 = { (int) off };
		Integer[] param4 = { (int) rtl };

		for (int i = 0; i < ins1.length; i++) {
			doPrivateMethod(msio, "changeDroneSts", cls1, param0);
			if (ins1[i] == dis || ins1[i] == amd) {
				doPrivateMethod(msio, "changeDroneSts", cls1, param12[i]);
			} else {
				if (ins2[i]) {
					doPrivateMethod(msio, "changeDroneSts", cls1, param3);
				} else if (ins3[i]) {
					doPrivateMethod(msio, "changeDroneSts", cls1, param4);
				}
			}
			assertEquals(
					doPrivateMethod(msio, "checkDroneSts", cls1, rets1[i]),
					true);
		}

	}

	@Test
	public void testFlightCtrlSelect() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		int off = (int) getPrivateField(msio, "STS_TAKEOFF");
		int mov = (int) getPrivateField(msio, "STS_IN_MOVE");
		int hov = (int) getPrivateField(msio, "STS_HOVERING");
		short fof = (short) getPrivateField(msio, "FM_TAKEOFF");
		short nod = (short) getPrivateField(msio, "FM_NO_DATA");

		Integer[] ins1 = { off, off, mov, mov, hov };
		Short[] ins2 = { fof, nod, null, null,
				null };
		Integer[] ins3 = { null, null, 1, 2, null };

		Integer[] ret1 = { hov };
		Integer[] ret2 = { 6 };
		Integer[][] rets1 = { ret2, ret1, ret1, ret2, ret2 };

		int[] retCnt1 = { 0, 1, 0, 0, 0 };
		int[] retCnt2 = { 0, 0, 1, 0, 0 };
		int j1 = 0;
		int j2 = 0;

		Class[] cls1 = { int.class };
		Integer[] param0 = { 6 };
		Integer[] param1 = { hov };


		for (int i = 0; i < ins1.length; i++) {
			doPrivateMethod(msio, "changeDroneSts", cls1, param0);
			j1 = 0;
			j2 = 0;
			if (ins1[i] == off) {
				if (ins2[i] != fof) {
					doPrivateMethod(msio, "changeDroneSts", cls1, param1);
					j1++;
				}
			} else if (ins1[i] == mov) {
				if (ins3[i] <= 1) {
					doPrivateMethod(msio, "changeDroneSts", cls1, param1);
					j2++;
				}
			}
			assertEquals(
					doPrivateMethod(msio, "checkDroneSts", cls1, rets1[i]),
					true);
			assertEquals(j1, retCnt1[i]);
			assertEquals(j2, retCnt2[i]);
		}

	}

	@Test
	public void testFlightTimeCount() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		setPrivateField(msio, "flight_time_start", 1000);
		long sta = System.currentTimeMillis() - 1000;

		int ret = (int) doPrivateMethod(msio, "flightTimeCount");
		assertEquals(ret, (int) sta / 1000);

	}

	@Test
	public void testPortWrite() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		int j = 0;

		msio.m_outGPSHomeOut.write();
		j++;
		msio.m_outGPSDroneOut.write();
		j++;
		msio.m_outBattVOut.write();
		j++;
		msio.m_outGyroOut.write();
		j++;
		msio.m_outAccelOut.write();
		j++;
		msio.m_outMagOut.write();
		j++;
		msio.m_outFlightTimeOut.write();
		j++;
		msio.m_outVehicleModeOut.write();
		j++;
		msio.m_outStateOut.write();
		j++;
		assertEquals(j, 9);
	}

	@Test
	public void testReadGPSHome() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Uav uav = mock(Uav.class);

		Class[] cls1 = { Uav.class };
		Uav[] param1 = { uav };

		Home hm = new Home();
		hm.setAlt(11.5);
		hm.setLat(12.5);
		hm.setLon(13.5);

		Home[] ins1 = { null, hm };

		GPSData gp = new GPSData();
		gp.altitude = 0.1;
		gp.latitude = 0.2;
		gp.longitude = 0.3;

		setPrivateField(msio, "home", gp);

		Double[] ret1 = { 0.2, 12.5 };
		Double[] ret2 = { 0.3, 13.5 };
		Double[] ret3 = { 0.1, 1.5 };

		for (int i = 0; i < ins1.length; i++) {
			when(uav.getHome()).thenReturn(ins1[i]);
			doPrivateMethod(msio, "readGPSHome", cls1, param1);
			GPSData home = (GPSData) getPrivateField(msio, "home");
			assertEquals(home.latitude == ret1[i], true);
			assertEquals(home.longitude == ret2[i], true);
			assertEquals(home.altitude == ret3[i], true);
		}

	}

	@Test
	public void testHomeAltGround() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Class[] cls1 = { Double.class };

		Double[] ins1 = { null };
		Double[] ins2 = { 11.5 };

		Double[][] param1 = { ins1, ins2 };

		Double[] ret1 = { null, 1.5 };

		for (int i = 0; i < param1.length; i++) {
			Double ret = (Double) doPrivateMethod(msio, "homeAltGround", cls1,
					param1[i]);
			assertEquals(ret, ret1[i]);
		}

	}

	@Test
	public void testSetGPSHome() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		GPSData gps = new GPSData();
		GPSTime gt = new GPSTime(0, 0);
		Time ti = new Time(0, 0);
		gps.altitude = 0.1;
		gps.fixType = RTC.GPSFixType.GPS_FIX_NORMAL;
		gps.heading = 0.0;
		gps.horizontalError = 0.0;
		gps.horizontalSpeed = 0.0;
		gps.latitude = 0.2;
		gps.longitude = 0.3;
		gps.numSatellites = 0;
		gps.timeFromGPS = gt;
		gps.tm = ti;
		gps.verticalError = 0.0;
		gps.verticalSpeed = 0.0;

		GPSData[] ins1 = { null };
		GPSData[] ins2 = { gps };
		Class[] cls1 = { GPSData.class };
		GPSData[][] param1 = { ins1, ins2 };

		double[] ret1 = { 0.0, 0.1 };
		RTC.GPSFixType[] ret2 = { null, RTC.GPSFixType.GPS_FIX_NORMAL };
		double[] ret3 = { 0.0, 0.0 };
		double[] ret4 = { 0.0, 0.0 };
		double[] ret5 = { 0.0, 0.0 };
		double[] ret6 = { 0.0, 0.2 };
		double[] ret7 = { 0.0, 0.3 };
		double[] ret8 = { 0.0, 0.0 };
		GPSTime[] ret9 = { null, gt };
		Time[] ret10 = { null, ti };
		double[] ret11 = { 0.0, 0.0 };
		double[] ret12 = { 0.0, 0.0 };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setGPSHome", cls1, param1[i]);

			assertEquals(msio.m_outGPSHome_val.altitude == ret1[i], true);
			assertEquals(msio.m_outGPSHome_val.fixType, ret2[i]);
			assertEquals(msio.m_outGPSHome_val.heading == ret3[i], true);
			assertEquals(msio.m_outGPSHome_val.horizontalError == ret4[i], true);
			assertEquals(msio.m_outGPSHome_val.horizontalSpeed == ret5[i], true);
			assertEquals(msio.m_outGPSHome_val.latitude == ret6[i], true);
			assertEquals(msio.m_outGPSHome_val.longitude == ret7[i], true);
			assertEquals(msio.m_outGPSHome_val.numSatellites == ret8[i], true);
			if (i == 0) {
				assertEquals(msio.m_outGPSHome_val.timeFromGPS, ret9[i]);
				assertEquals(msio.m_outGPSHome_val.tm, ret10[i]);
			} else {
				assertEquals(msio.m_outGPSHome_val.timeFromGPS.msec,
						ret9[i].msec);
				assertEquals(msio.m_outGPSHome_val.timeFromGPS.sec, ret9[i].sec);
				assertEquals(msio.m_outGPSHome_val.tm.nsec, ret10[i].nsec);
				assertEquals(msio.m_outGPSHome_val.tm.sec, ret10[i].sec);
			}
			assertEquals(msio.m_outGPSHome_val.verticalError == ret11[i], true);
			assertEquals(msio.m_outGPSHome_val.verticalSpeed == ret12[i], true);

		}
	}

	@Test
	public void testReadGPSDrone() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Uav uav = mock(Uav.class);

		Class[] cls1 = { Uav.class };
		Uav[] param1 = { uav };

		Gps hm = new Gps();
		hm.setAlt(11.5);
		hm.setLat(12.5);
		hm.setLon(13.5);

		Gps[] ins1 = { null, hm };

		GPSData gp = new GPSData();
		gp.altitude = 0.1;
		gp.latitude = 0.2;
		gp.longitude = 0.3;

		setPrivateField(msio, "drone", gp);

		Double[] ret1 = { 0.2, 12.5 };
		Double[] ret2 = { 0.3, 13.5 };
		Double[] ret3 = { 0.1, 11.5 };

		for (int i = 0; i < ins1.length; i++) {
			when(uav.getGPS()).thenReturn(ins1[i]);
			doPrivateMethod(msio, "readGPSDrone", cls1, param1);
			GPSData drone = (GPSData) getPrivateField(msio, "drone");
			assertEquals(drone.latitude == ret1[i], true);
			assertEquals(drone.longitude == ret2[i], true);
			assertEquals(drone.altitude == ret3[i], true);
		}

	}

	@Test
	public void testSetGPSDrone() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		GPSData gps = new GPSData();
		GPSTime gt = new GPSTime(0, 0);
		Time ti = new Time(0, 0);
		gps.altitude = 0.1;
		gps.fixType = RTC.GPSFixType.GPS_FIX_NORMAL;
		gps.heading = 0.0;
		gps.horizontalError = 0.0;
		gps.horizontalSpeed = 0.0;
		gps.latitude = 0.2;
		gps.longitude = 0.3;
		gps.numSatellites = 0;
		gps.timeFromGPS = gt;
		gps.tm = ti;
		gps.verticalError = 0.0;
		gps.verticalSpeed = 0.0;

		GPSData[] ins1 = { null };
		GPSData[] ins2 = { gps };
		Class[] cls1 = { GPSData.class };
		GPSData[][] param1 = { ins1, ins2 };

		double[] ret1 = { 0.0, 0.1 };
		RTC.GPSFixType[] ret2 = { null, RTC.GPSFixType.GPS_FIX_NORMAL };
		double[] ret3 = { 0.0, 0.0 };
		double[] ret4 = { 0.0, 0.0 };
		double[] ret5 = { 0.0, 0.0 };
		double[] ret6 = { 0.0, 0.2 };
		double[] ret7 = { 0.0, 0.3 };
		double[] ret8 = { 0.0, 0.0 };
		GPSTime[] ret9 = { null, gt };
		Time[] ret10 = { null, ti };
		double[] ret11 = { 0.0, 0.0 };
		double[] ret12 = { 0.0, 0.0 };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setGPSDrone", cls1, param1[i]);

			assertEquals(msio.m_outGPSDrone_val.altitude == ret1[i], true);
			assertEquals(msio.m_outGPSDrone_val.fixType == ret2[i], true);
			assertEquals(msio.m_outGPSDrone_val.heading == ret3[i], true);
			assertEquals(msio.m_outGPSDrone_val.horizontalError == ret4[i],
					true);
			assertEquals(msio.m_outGPSDrone_val.horizontalSpeed == ret5[i],
					true);
			assertEquals(msio.m_outGPSDrone_val.latitude == ret6[i], true);
			assertEquals(msio.m_outGPSDrone_val.longitude == ret7[i], true);
			assertEquals(msio.m_outGPSDrone_val.numSatellites == ret8[i], true);
			if (i == 0) {
				assertEquals(msio.m_outGPSDrone_val.timeFromGPS, ret9[i]);
				assertEquals(msio.m_outGPSDrone_val.tm, ret10[i]);
			} else {
				assertEquals(msio.m_outGPSDrone_val.timeFromGPS.msec,
						ret9[i].msec);
				assertEquals(msio.m_outGPSDrone_val.timeFromGPS.sec,
						ret9[i].sec);
				assertEquals(msio.m_outGPSDrone_val.tm.nsec, ret10[i].nsec);
				assertEquals(msio.m_outGPSDrone_val.tm.sec, ret10[i].sec);
			}
			assertEquals(msio.m_outGPSDrone_val.verticalError == ret11[i], true);
			assertEquals(msio.m_outGPSDrone_val.verticalSpeed == ret12[i], true);

		}
	}

	@Test
	public void testReadVoltage() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Uav uav = mock(Uav.class);

		Class[] cls1 = { Uav.class };
		Uav[] param1 = { uav };

		Double[] ins1 = { null, 1.1 };

		Float[] ret1 = { (float) 0.0, (float) 1.1 };

		for (int i = 0; i < ins1.length; i++) {
			when(uav.getVoltage()).thenReturn(ins1[i]);
			TimedFloat ret = (TimedFloat) doPrivateMethod(msio, "readVoltage",
					cls1, param1);
			assertEquals(ret.data == ret1[i], true);
		}

	}

	@Test
	public void testSetVoltage() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		TimedFloat tf = new TimedFloat();
		Time ti = new Time(0, 0);
		tf.data = (float) 1.1;
		tf.tm = ti;

		TimedFloat[] ins1 = { null };
		TimedFloat[] ins2 = { tf };
		Class[] cls1 = { TimedFloat.class };
		TimedFloat[][] param1 = { ins1, ins2 };

		float[] ret1 = { (float) 0.0, (float) 1.1 };
		Time[] ret2 = { null, ti };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setVoltage", cls1, param1[i]);

			assertEquals(msio.m_outBattV_val.data == ret1[i], true);
			if (i == 0) {
				assertEquals(msio.m_outBattV_val.tm, ret2[i]);
			} else {
				assertEquals(msio.m_outBattV_val.tm.nsec, ret2[i].nsec);
				assertEquals(msio.m_outBattV_val.tm.sec, ret2[i].sec);
			}

		}
	}

	@Test
	public void testSetFlightTime() throws Exception {
		Time ecpected = new Time();
		ecpected.sec = 1;
		ecpected.nsec = 0;
		Method method = MiniSV06LAImpl.class.getDeclaredMethod(
				"setFlightTime", int.class);
		method.setAccessible(true);

		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Field fld = MiniSV06LAImpl.class
				.getDeclaredField("m_outFlightTime_val");
		fld.setAccessible(true);

		method.invoke(msio, 1);

		assertEquals(ecpected.sec,
				((Time) getPrivateField(msio, "m_outFlightTime_val")).sec);
		assertEquals(ecpected.nsec,
				((Time) getPrivateField(msio, "m_outFlightTime_val")).nsec);
	}

	@Test
	public void testReadGyro() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Uav uav = mock(Uav.class);
		Imu imu = new Imu();
		imu.setMagNorm(1.1);

		Class[] cls1 = { Uav.class };
		Uav[] param1 = { uav };

		Imu[] ins1 = { null, imu, imu, imu, imu };
		List<Double> ls = new ArrayList();
		ls.add(1.2);
		ls.add(2.2);
		ls.add(3.2);
		List[] ins2 = { null, null, null, ls, ls };
		Integer[] ins3 = { null, 1, 3, 1, 3 };

		TimedOrientation3D gyro = new TimedOrientation3D();
		TimedOrientation3D ret = gyro;

		double ret1 = 1.1;
		double ret2 = 1.2;
		double ret3 = 1.3;

		for (int i = 0; i < ins1.length; i++) {
			gyro.data = new Orientation3D();
			if (ins1[i] != null) {
				if (ins2[i] != null && ins3[i] == 3) {
					gyro.data.r = 1.1;
					gyro.data.p = 1.2;
					gyro.data.y = 1.3;
				}
			}
			if (i == 4) {
				assertEquals(gyro.data.r == ret1, true);
				assertEquals(gyro.data.p == ret2, true);
				assertEquals(gyro.data.y == ret3, true);
			} else {
				assertEquals(gyro.data.r == ret.data.r, true);
				assertEquals(gyro.data.p == ret.data.p, true);
				assertEquals(gyro.data.y == ret.data.y, true);
			}
		}

	}

	@Test
	public void testSetGyro() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		TimedOrientation3D gyro = new TimedOrientation3D();
		Time ti = new Time(0, 0);
		gyro.data = new Orientation3D();
		gyro.data.r = 1.1;
		gyro.data.p = 1.2;
		gyro.data.y = 1.3;
		gyro.tm = ti;

		TimedOrientation3D[] ins1 = { null };
		TimedOrientation3D[] ins2 = { gyro };
		Class[] cls1 = { TimedOrientation3D.class };
		TimedOrientation3D[][] param1 = { ins1, ins2 };

		Orientation3D[] ret1 = { null, gyro.data };
		Time[] ret2 = { null, ti };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setGyro", cls1, param1[i]);

			assertEquals(msio.m_outGyro_val.data, ret1[i]);
			if (i == 0) {
				assertEquals(msio.m_outGyro_val.tm, ret2[i]);
			} else {
				assertEquals(msio.m_outGyro_val.tm.nsec, ret2[i].nsec);
				assertEquals(msio.m_outGyro_val.tm.sec, ret2[i].sec);
			}

		}
	}

	@Test
	public void testReadAccel() throws Exception {
		Imu imu = new Imu();
		imu.setMagNorm(1.1);

		Imu[] ins1 = { null, imu, imu, imu, imu };
		List<Double> ls = new ArrayList();
		ls.add(1.2);
		ls.add(2.2);
		ls.add(3.2);
		List[] ins2 = { null, null, null, ls, ls };
		Integer[] ins3 = { null, 1, 3, 1, 3 };

		TimedFloatSeq accel = new TimedFloatSeq();
		TimedFloatSeq ret = accel;

		float ret1 = (float) 1.1;
		float ret2 = (float) 1.2;
		float ret3 = (float) 1.3;

		for (int i = 0; i < ins1.length; i++) {
			accel.data = new float[] { 1, 1, 1 };
			if (ins1[i] != null) {
				if (ins2[i] != null && ins3[i] == 3) {
					accel.data[0] = MathConv.nullTo0(1.1).floatValue();
					accel.data[1] = MathConv.nullTo0(1.2).floatValue();
					accel.data[2] = MathConv.nullTo0(1.3).floatValue();
				}
			}
			if (i == 4) {
				assertEquals(accel.data[0] == ret1, true);
				assertEquals(accel.data[1] == ret2, true);
				assertEquals(accel.data[2] == ret3, true);
			} else {
				assertEquals(accel.data[0] == ret.data[0], true);
				assertEquals(accel.data[1] == ret.data[1], true);
				assertEquals(accel.data[2] == ret.data[2], true);
			}
		}

	}

	@Test
	public void testSetAccel() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		TimedFloatSeq tf = new TimedFloatSeq();
		Time ti = new Time(0, 0);
		float[] fl = new float[] { (float) 1.1 };
		tf.data = fl;
		tf.tm = ti;

		TimedFloatSeq[] ins1 = { null };
		TimedFloatSeq[] ins2 = { tf };
		Class[] cls1 = { TimedFloatSeq.class };
		TimedFloatSeq[][] param1 = { ins1, ins2 };

		float[][] ret1 = { null, fl };
		Time[] ret2 = { null, ti };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setAccel", cls1, param1[i]);

			assertEquals(msio.m_outAccel_val.data == ret1[i], true);
			if (i == 0) {
				assertEquals(msio.m_outAccel_val.tm, ret2[i]);
			} else {
				assertEquals(msio.m_outAccel_val.tm.nsec, ret2[i].nsec);
				assertEquals(msio.m_outAccel_val.tm.sec, ret2[i].sec);
			}

		}
	}

	@Test
	public void testReadMag() throws Exception {
		Uav uav = mock(Uav.class);
		Imu imu = new Imu();
		imu.setMagNorm(1.1);

		Imu[] ins1 = { null, imu, imu, imu, imu };
		List<Double> ls = new ArrayList();
		ls.add(1.2);
		ls.add(2.2);
		ls.add(3.2);
		List[] ins2 = { null, null, null, ls, ls };
		Integer[] ins3 = { null, 1, 3, 1, 3 };

		TimedFloat mag = new TimedFloat();
		TimedFloat ret = mag;

		float ret1 = (float) 1.1;

		for (int i = 0; i < ins1.length; i++) {
			mag.data = 1;
			if (ins1[i] != null) {
				if (ins2[i] != null && ins3[i] == 3) {
					mag.data = MathConv.nullTo0(1.1).floatValue();
				}
			}
			if (i == 4) {
				assertEquals(mag.data == ret1, true);
			} else {
				assertEquals(mag.data == ret.data, true);
			}
		}

	}

	@Test
	public void testSetMag() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		TimedFloat tf = new TimedFloat();
		Time ti = new Time(0, 0);
		tf.data = (float) 1.1;
		tf.tm = ti;

		TimedFloat[] ins1 = { null };
		TimedFloat[] ins2 = { tf };
		Class[] cls1 = { TimedFloat.class };
		TimedFloat[][] param1 = { ins1, ins2 };

		float[] ret1 = { (float) 0.0, (float) 1.1 };
		Time[] ret2 = { null, ti };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setMag", cls1, param1[i]);

			assertEquals(msio.m_outMag_val.data == ret1[i], true);
			if (i == 0) {
				assertEquals(msio.m_outMag_val.tm, ret2[i]);
			} else {
				assertEquals(msio.m_outMag_val.tm.nsec, ret2[i].nsec);
				assertEquals(msio.m_outMag_val.tm.sec, ret2[i].sec);
			}

		}
	}

	@Test
	public void testReadVehicleMode() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Boolean[] ins1 = { false, true, true, true, true };
		Boolean[] ins2 = { null, false, true, true, true };
		Boolean[] ins3 = { null, null, false, true, true };
		Short[] ins4 = { null, null, null, 1, 4 };

		Short[] ins5 = { null, null, null, 1, 4 };
		Short[] ins6 = { null, null, null, null, 1 };

		TimedFloat mag = new TimedFloat();
		Short[] ret1 = { null, null, null, 1, 1 };
		String[] ret2 = { null, null, null, "Manual", "Navi(Command)" };

		Short vehicle_state = (Short) getPrivateField(msio, "vehicle_state");
		TimedString vehicle = (TimedString) getPrivateField(msio, "vehicle");

		Class[] cls1 = { Short.class };

		for (int i = 0; i < ins1.length; i++) {
			vehicle_state = null;
			vehicle.data = null;
			if (ins1[i]) {
				if (ins2[i]) {
					if (ins3[i]) {
						vehicle_state = ins5[i];
						vehicle.data = (String) doPrivateMethod(msio,
								"selectVehicleCtrl", cls1,
								new Short[] { vehicle_state });
						if (ins4[i].equals(getPrivateField(msio, "FM_NAVI"))) {
							vehicle_state = ins6[i];
							vehicle.data = (String) doPrivateMethod(msio,
									"selectVehicleAuto", cls1,
									new Short[] { vehicle_state });
						}
					}
				}
			}
			assertEquals(vehicle_state, ret1[i]);
			assertEquals(vehicle.data, ret2[i]);

		}

	}

	@Test
	public void testSelectVehicleCtrl() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Short nod = (Short) getPrivateField(msio, "FM_NO_DATA");
		Short man = (Short) getPrivateField(msio, "FM_MANUAL");
		Short att = (Short) getPrivateField(msio, "FM_ATT");
		Short gps = (Short) getPrivateField(msio, "FM_GPSA");
		Short nav = (Short) getPrivateField(msio, "FM_NAVI");
		Short off = (Short) getPrivateField(msio, "FM_TAKEOFF");
		Short lnd = (Short) getPrivateField(msio, "FM_LAND");

		Short[] ins1 = { nod };
		Short[] ins2 = { man };
		Short[] ins3 = { att };
		Short[] ins4 = { gps };
		Short[] ins5 = { nav };
		Short[] ins6 = { off };
		Short[] ins7 = { lnd };

		Class[] cls1 = { Short.class };
		Short[][] param1 = { ins1, ins2, ins3, ins4, ins5, ins6, ins7 };

		String[] ret1 = { "NO_DATA", "Manual", "Assist(Gyro)", "Assist(GPS)",
				"Navi", "Auto-takeoff", "Auto-landing" };

		for (int i = 0; i < param1.length; i++) {
			String ret = (String) doPrivateMethod(msio, "selectVehicleCtrl",
					cls1, param1[i]);

			assertEquals(ret, ret1[i]);

		}

	}

	@Test
	public void testSelectVehicleAuto() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Short nod = (Short) getPrivateField(msio, "FM_NO_DATA");
		Short man = (Short) getPrivateField(msio, "FM_COMMAND");
		Short att = (Short) getPrivateField(msio, "FM_PLAN");
		Short gps = (Short) getPrivateField(msio, "FM_GOHOME");

		Short[] ins1 = { nod };
		Short[] ins2 = { man };
		Short[] ins3 = { att };
		Short[] ins4 = { gps };

		Class[] cls1 = { Short.class };
		Short[][] param1 = { ins1, ins2, ins3, ins4 };

		String[] ret1 = { "NAVI", "Navi(Command)", "Plan", "Navi(GoHome)" };

		for (int i = 0; i < param1.length; i++) {
			String ret = (String) doPrivateMethod(msio, "selectVehicleAuto",
					cls1, param1[i]);

			assertEquals(ret, ret1[i]);

		}

	}

	@Test
	public void testSetVehicleMode() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		TimedString vehicle = new TimedString();
		vehicle.data = "NAVI";

		Class[] cls1 = { TimedString.class };
		TimedString[] ins1 = { null };
		TimedString[] ins2 = { vehicle };
		TimedString[][] param1 = { ins1, ins2 };

		String[] ret1 = { null, vehicle.data };
		Time[] ret2 = { null, new Time(0, 0) };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setVehicleMode", cls1, param1[i]);

			assertEquals(msio.m_outVehicleMode_val.data, ret1[i]);
			if (i == 0) {
				assertEquals(msio.m_outVehicleMode_val.tm, ret2[i]);
			} else {
				assertEquals(msio.m_outVehicleMode_val.tm.nsec, ret2[i].nsec);
				assertEquals(msio.m_outVehicleMode_val.tm.sec, ret2[i].sec);
			}
		}

	}

	@Test
	public void testSetState() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		String state = "NAVI";

		Class[] cls1 = { String.class };
		String[] ins1 = { state };
		String[][] param1 = { ins1 };

		String[] ret1 = { state };
		Time[] ret2 = { new Time(0, 0) };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setState", cls1, param1[i]);

			assertEquals(msio.m_outState_val.data, ret1[i]);
			assertEquals(msio.m_outState_val.tm.nsec, ret2[i].nsec);
			assertEquals(msio.m_outState_val.tm.sec, ret2[i].sec);

		}

	}

	@Test
	public void testSetSysErr() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		String state = "NAVI";

		Class[] cls1 = { String.class };
		String[] ins1 = { state };
		String[][] param1 = { ins1 };

		String[] ret1 = { state };
		Time[] ret2 = { new Time(0, 0) };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setSysErr", cls1, param1[i]);

			assertEquals(msio.m_outSysErr_val.data, ret1[i]);
			assertEquals(msio.m_outSysErr_val.tm.nsec, ret2[i].nsec);
			assertEquals(msio.m_outSysErr_val.tm.sec, ret2[i].sec);

		}

	}

	@Test
	public void testReadDrv() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		Uav uav = new Uav();
		Sys sys = new Sys();
		ms.Mode mode = new ms.Mode();

		mode.setDrv((short) 1);
		sys.setMode(mode);

		Class[] cls1 = { Uav.class };
		Sys[] ins1 = { null, sys };
		Uav[] param1 = { uav };

		Short[] ret1 = { 0, 1 };

		for (int i = 0; i < ins1.length; i++) {
			uav.setSys(ins1[i]);
			Short ret = (Short) doPrivateMethod(msio, "readDrv", cls1,
					new Uav[] { uav });
			assertEquals(ret, ret1[i]);
		}

	}

	@Test
	public void testSetDrv() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		Short state = 1;
		Short drv = (Short) getPrivateField(msio, "drv_state");
		Class[] cls1 = { Short.class };
		Short[] ins1 = { drv };
		Short[] ins2 = { state };
		Short[][] param1 = { ins1, ins2 };

		Short[] ret1 = { drv, state };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "setDrv", cls1, param1[i]);

			assertEquals((Short) getPrivateField(msio, "drv_state"), ret1[i]);

		}

	}

	@Test
	public void testErrMsgWrite() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		String state = "test";
		Class[] cls1 = { String.class };
		String[] ins1 = { state };
		String[][] param1 = { ins1 };

		for (int i = 0; i < param1.length; i++) {
			doPrivateMethod(msio, "errMsgWrite", cls1, param1[i]);
			assertEquals(msio.m_outSysErr_val.data, state);
		}

	}

	@Test
	public void testStartSleep() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);
		try {
			Thread.sleep(1000);
			throw new InterruptedException();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	@Test
	public void testChangeDroneSts() throws Exception {
		Manage mg = new Manage();
		MiniSV06LAImpl msio = new MiniSV06LAImpl(mg);

		doPrivateMethod(msio, "changeDroneSts", new Class[] { int.class },
				new Integer[] { 1 });

		assertEquals(getPrivateField(msio, "drone_state"), "Armed");
		assertEquals(msio.m_outState_val.data, "Armed");
	}

}

class Manage extends Manager {

}
